#! /usr/local/bin/perl -w

use strict;

use POSIX;
use Digest::MD5 qw(md5_hex);

my $testcnt = 0;
my $testfailed = 0;

sub runtest {
    my $cmd = shift;
    my $expected = shift;
    my $buf = "";
    $testcnt++;
    printf "Running test %d: %s\n", $testcnt, join(" ", @$cmd);
    open PROG, "-|", @$cmd;
    while (<PROG>) {
	print ">> $_";
	$buf .= $_;
    }
    close PROG;
    if ( md5_hex($buf) eq $expected ) {
	print "ok\n";
    } else {
	print "FAILED!\n";
	$testfailed++;
    }
}

sub copysuid {
    my $prgname = shift;
    print "making suid root version of $prgname...\n";
    system "cp", "$prgname", "$prgname.suid";
    system "chmod", "u+s", "$prgname.suid";
}

sub delsuid {
    my $prgname = shift;
    print "removing suid root version of $prgname...\n";
    unlink "$prgname.suid";
}

$< = 0;
$> = 0;
die "Please run this as root" unless ($<==0 && $>==0);
runtest ["./uidtest"], "52b3369df7b7d87c27a3b6d24f286536";
copysuid "uidtest";
runtest ["./uidtest.suid"], "187f794342f7f45d5a7220e9f39b9812";
delsuid "uidtest";
runtest ["./captest"], "a18c1db3cdd41f96436ef45555397bec";
copysuid "captest";
runtest ["./captest.suid"], "9e06c1092bdc14838706febabf939de9";
delsuid "captest";
runtest ["./captest1"], "2453e49d5ad21829bdf510728b7c7984";
runtest ["./captest2"], "efc0d4cbd84b41b229c38caa2407f051";
copysuid "captest2";
runtest ["./captest2.suid"], "8ba15ff3b69d88955c69df9e2b8d67c8";
delsuid "captest2";
if ($testfailed) {
    print "$testfailed/$testcnt tests failed\n";
} else {
    print "all tests ok\n";
}
exit ($testfailed != 0);
