#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "capability.h"
#include "testsuite.h"

void
print_uids (void)
{
  uid_t uid, euid;

  uid = getuid ();
  euid = geteuid ();
  printf ("uid=%ld, euid=%ld\n", (long int)uid, (long int)euid);
}

void
print_caps (uint64_t *prm, uint64_t *eff, uint64_t *inh)
{
  struct user_cap_header_struct header;
  struct user_cap_data_struct data;

  header.version = _LINUX_CAPABILITY_VERSION;
  header.pid = getpid ();
  if ( capget (&header, &data) < 0 )
    {
      perror ("capget");
      exit (EXIT_FAILURE);
    }
  printf ("prm=0x%016llx, eff=0x%016llx, inh=0x%016llx\n",
	  (unsigned long long)(data.permitted),
	  (unsigned long long)(data.effective),
	  (unsigned long long)(data.inheritable));
  if ( prm )
    *prm = data.permitted;
  if ( eff )
    *eff = data.effective;
  if ( inh )
    *inh = data.inheritable;
}

void
set_caps (uint64_t prm, uint64_t eff, uint64_t inh)
{
  struct user_cap_header_struct header;
  struct user_cap_data_struct data;

  header.version = _LINUX_CAPABILITY_VERSION;
  header.pid = getpid ();
  data.permitted = prm;
  data.effective = eff;
  data.inheritable = inh;
  capset (&header, &data);
}

void
fork_and_print (const char *prgname)
{
  pid_t child;

  fflush (stdout);
  child = fork ();
  if ( child < 0 )
    perror ("fork");
  else if ( child )
    waitpid (child, NULL, 0);
  else
    {
      printf ("forked: ");
      fflush (stdout);
      execlp (prgname, prgname, "print", NULL);
      perror ("exec");
      exit (EXIT_FAILURE);
    }
}
